package com.example;

/**
 * Dispatches over different models.
 * 
 * @author nikostotz
 * 
 */
public interface IPerformanceTestDispatcher {
	enum AccessType {
		/**
		 * Access to generic list (i. e. AbstractEntry)
		 */
		GENERIC,

		/**
		 * Access to specific list (i. e. Book)
		 */
		SPECIFIC
	}

	void setAccessType(AccessType accessType);

	/**
	 * Count by accessor method.
	 * 
	 * <p>
	 * Generic:
	 * 
	 * <pre>
	 * return library.getEntries().size();
	 * </pre>
	 * 
	 * </p>
	 * 
	 * <p>
	 * Specific:
	 * 
	 * <pre>
	 * return library.getBooks().size();
	 * </pre>
	 * 
	 * </p>
	 */
	void computeStatistics();

	/**
	 * Count by checking type of complement accessor.
	 * 
	 * <p>
	 * Generic:
	 * 
	 * <pre>
	 * if (library instanceof BookLibrary) {
	 * 	return ((BookLibrary) library).getBooks().size();
	 * } else if (library instanceof MovieLibrary) {
	 * 	return ((MovieLibrary) library).getMovies().size();
	 * } else {
	 * 	throw new MetamodelException(library);
	 * }
	 * </pre>
	 * 
	 * </p>
	 * 
	 * <p>
	 * Specific:
	 * 
	 * <pre>
	 * return Iterables.size(Iterables.filter(library.getEntries(), Book.class));
	 * </pre>
	 * 
	 * </p>
	 */
	void computeInstanceof();

	/**
	 * Count by using Xtend2 <code>dispatch</code>.
	 * 
	 * <p>
	 * Generic:
	 * 
	 * <pre>
	 * def dispatch countEntries(BookLibrary library) {
	 * 	return library.getBooks().size()
	 * }
	 * 
	 * def dispatch countEntries(MovieLibrary library) {
	 * 	return library.getMovies().size()
	 * }
	 * </pre>
	 * 
	 * </p>
	 * 
	 * <p>
	 * Specific:
	 * 
	 * <pre>
	 * override countBooks(BookLibrary library) {
	 * 	return library.getBooks().size()
	 * }
	 * </pre>
	 * 
	 * </p>
	 */
	void computeDispatch();
}
